"use strict";
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
const camelcase_1 = __importDefault(require("camelcase"));
const utils_1 = require("../utils");
const baseTypes = ['string', 'number', 'object', 'boolean', 'any'];
/** 类模板 */
function interfaceTemplate(name, props, imports, strictNullChecks = true) {
    // 所有的引用
    const importString = imports
        .map(imp => {
        return `import { ${imp} } from '../definitions/${imp}'\n`;
    })
        .join('');
    return `
  ${importString}

  export interface ${name} {

    ${props.map(p => classPropsTemplate(p.name, p.type, p.format, p.desc, !strictNullChecks, false, false)).join('')}
  }
  `;
}
exports.interfaceTemplate = interfaceTemplate;
/** 类模板 */
function classTemplate(name, props, imports, strictNullChecks = true, useClassTransformer, generateValidationModel) {
    // 所有的引用
    const mappedImports = imports.map(imp => {
        return `import { ${imp} } from '../definitions/${imp}'\n`;
    });
    if (useClassTransformer && imports.length > 0) {
        mappedImports.push(`import { Type, Transform, Expose } from 'class-transformer'\n`);
    }
    const importString = mappedImports.join('');
    return `
  ${importString}

  export class ${name} {

    ${props
        .map(p => classPropsTemplate(p.name, p.type, p.format, p.desc, !strictNullChecks, useClassTransformer, p.isEnum || p.isType))
        .join('')}

    constructor(data: (undefined | any) = {}){
        ${props.map(p => classConstructorTemplate(p.name)).join('')}
    }
    ${generateValidationModel ? classValidationModelTemplate(props) : ''}
  }
  `;
}
exports.classTemplate = classTemplate;
/** 类属性模板 */
function classPropsTemplate(filedName, type, format, description, canNull, useClassTransformer, isType) {
    /**
     * eg:
     *   //description
     *   fieldName: type
     */
    type = utils_1.toBaseType(type, format);
    if (useClassTransformer) {
        const decorators = classTransformTemplate(type, format, isType);
        return `
  /** ${description || ''} */
  ${decorators}
  '${filedName}'${canNull ? '?' : ''}:${type};
  `;
    }
    else {
        return `
  /** ${description || ''} */
  '${filedName}'${canNull ? '?' : ''}:${type};
  `;
    }
}
exports.classPropsTemplate = classPropsTemplate;
function propValidationModelTemplate(filedName, validationModel) {
    /**
     * eg:
     *   fieldName: { required: true, maxLength: 50 }
     */
    return `'${filedName}':${JSON.stringify(validationModel)}`;
}
exports.propValidationModelTemplate = propValidationModelTemplate;
function classValidationModelTemplate(props) {
    /**
     * eg:
     *   public static validationModel = { .. }
     */
    return `
    public static validationModel = {
      ${props
        .filter(p => p.validationModel !== null)
        .map(p => propValidationModelTemplate(p.name, p.validationModel))
        .join(',\n')}
    }
  `;
}
exports.classValidationModelTemplate = classValidationModelTemplate;
function classTransformTemplate(type, format, isType) {
    const decorators = [`@Expose()`];
    const nonArrayType = type.replace('[', '').replace(']', '');
    /* ignore interfaces */
    if (baseTypes.indexOf(nonArrayType) < 0 && !isType) {
        decorators.push(`@Type(() => ${nonArrayType})`);
    }
    return decorators.join('\n');
}
exports.classTransformTemplate = classTransformTemplate;
/** 类属性模板 */
function classConstructorTemplate(name) {
    return `this['${name}'] = data['${name}'];\n`;
}
exports.classConstructorTemplate = classConstructorTemplate;
/** 枚举 */
function enumTemplate(name, enumString, prefix) {
    return `
  export enum ${name}{
    ${enumString}
  }
  `;
}
exports.enumTemplate = enumTemplate;
function typeTemplate(name, typeString, prefix) {
    return `
  export type ${name} = ${typeString};
  `;
}
exports.typeTemplate = typeTemplate;
/** requestTemplate */
function requestTemplate(name, requestSchema, options) {
    let { summary = '', parameters = '', responseType = '', method = '', path = '', } = requestSchema;
    path = path.replace(/{[^}]+}/, '');
    return `
/**
 * ${summary || ''}
 */
${options.useStaticMethod ? 'static' : ''} ${camelcase_1.default(name)}(${parameters}):Observable<${responseType}> {
  let url = '${path}'
  ${optionsStr(method, parameters)}
  return this.http.request("${method}", url, options) as any as Observable<${responseType}>;
}`;
}
exports.requestTemplate = requestTemplate;
/** serviceTemplate */
function serviceTemplate(name, body) {
    return `
  @Injectable({ providedIn: 'root' })
  export class ${name} {
      constructor(private http: HttpClient) {}
      ${body}
  } 
  `;
}
exports.serviceTemplate = serviceTemplate;
function optionsStr(method, parameters) {
    if (method == "get") {
        if (parameters)
            return `const _copy: any = {...params}
      let options : any = {
          params: new HttpParams({ fromObject: _copy }),
          method: "${method}"
      };`;
        else
            return `let options : any = {
        method: "${method}"
      };`;
    }
    else {
        return `let options : any = {
      body: params.body,
      method: "${method}"
      };`;
    }
}
exports.optionsStr = optionsStr;
//# sourceMappingURL=template.js.map